@@ -1,5 +1,5 @@
 # -*- mode:Perl -*-
-# $Id: Whois.pm,v 1.4 1997/05/23 03:40:00 chip Beta $
+# $Id: Whois.pm,v 1.13 1999/01/30 03:00:24 chip Exp $
 
 package Net::Whois;
 BEGIN { require 5.003 }
@@ -32,6 +32,13 @@ Net::Whois - Get and parse "whois" data from InterNIC
         }
     }
 
+    print "Domain Status:    ", $w->status,           "\n"
+        if $w->status;
+    print "Record Created:   ", $w->record_created,   "\n";
+        if $w->record_created;;
+    print "Record Updated:   ", $w->record_updated,   "\n";
+    print "Database Updated: ", $w->database_updated, "\n";
+
     $cur_server = Net::Whois::server;
     Net::Whois::server 'new.whois.server';  # optional
 
@@ -66,7 +73,7 @@ use IO::Socket;
 use Carp;
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
-$VERSION = '0.21';
+$VERSION = '0.23';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -184,22 +191,33 @@ sub new {
     my $class = @_ ? shift : 'Net::Whois';
     @_ == 1 or croak "usage: new $class DOMAIN";
     my ($domain) = @_;
+    my $text;
 
     my $sock = Net::Whois::_connect();
     print $sock "dom $domain\r\n";
-    my $text;
-    { undef $/; $text = <$sock> }
-    $sock->close;
-
+    { local $/; $text = <$sock> }
+    undef $sock;
     $text || return;
 
+    if ($text =~ /single out one record/) {
+	return unless $text =~ /\((.+?)\)[ \t]+\Q$domain\E\r?\n/i;
+	my $newdomain = $1;
+	$sock = Net::Whois::_connect();
+	print $sock "dom $newdomain\r\n";
+	{ local $/; $text = <$sock> }
+	undef $sock;
+	$text || return;
+    }
+
     $text =~ s/^ +//gm;
     my @text = split / *\r?\n/, $text;
-    my ($t, @t);
+    my (@t, $t, $c);
 
     my %info;
 
-    $t = shift @text;
+    do {
+	$t = shift @text;
+    } while (@text && ($t eq '' || $t =~ /^registrant:/i));
     @info{'NAME', 'TAG'} = ($t =~ /^(.*)\s+\((\S+)\)$/)
 	or return;
 
@@ -217,9 +235,9 @@ sub new {
 	pop @t;
 	$t = uc $t;
     }
-    elsif (my $code = country2code($t)) {
+    elsif ($c = country2code($t)) {
 	pop @t;
-	$t = uc $code;
+	$t = uc $c;
     }
     elsif ($t =~ /,\s*([^,]+?)(?:\s+\d{5}(?:-\d{4})?)?$/) {
 	$t = $US_State{uc $1} ? 'US' : undef;
@@ -242,19 +260,23 @@ sub new {
 		last if $text[0] =~ /contact.*:$/i;
 		push @c, shift @text;
 	    }
-	    @{$info{CONTACTS}}{map {uc} @ctypes} = (\@c) x @ctypes;
+	    @{$info{CONTACTS}}{map { uc $_ } @ctypes} = (\@c) x @ctypes;
 	}
-	elsif ($t =~ /^record created on (\S+)\.$/) {
-	    $info{CREATED} = $1;
+	elsif ($t =~ /^domain status:\s+(.*)$/i) {
+	    $info{STATUS} = $1;
 	}
-	elsif ($t =~ /^record last updated on (\S+)\.$/) {
-	    $info{UPDATED} = $1;
+	elsif ($t =~ /^record created on (.*)\.$/i) {
+	    $info{RECORD_CREATED} = $1;
+	}
+	elsif ($t =~ /^(record|database) last updated on (.*)\.$/i) {
+	    $info{"\U$1_UPDATED"} = $2;
 	}
 	elsif ($t =~ /^domain servers/i) {
 	    my @s;
 	    shift @text unless $text[0];
 	    while ($t = shift @text) {
-		push @s, [split /\s+/, $t];
+		next unless $t =~ s/\s*(\d{1,3}(?:\.\d{1,3}){3})$//;
+		push @s, [$t, $1];
 	    }
 	    $info{SERVERS} = \@s;
 	}
@@ -294,6 +316,11 @@ sub contacts {
     $self->[0]->{CONTACTS};
 }
 
+sub status {
+    my $self = shift;
+    $self->[0]->{STATUS};
+}
+
 sub servers {
     my $self = shift;
     $self->[0]->{SERVERS};
@@ -309,4 +336,9 @@ sub record_updated {
     $self->[0]->{RECORD_UPDATED};
 }
 
+sub database_updated {
+    my $self = shift;
+    $self->[0]->{DATABASE_UPDATED};
+}
+
 1;
